@@ -1,9 +1,7 @@
 maxBFGS <- function(fn, grad=NULL, hess=NULL,
                     start, fixed = NULL,
-                    print.level=0,
-                    iterlim=200,
+                    control=NULL,
                     constraints=NULL,
-                    tol=1e-8, reltol=tol,
                     finalHessian=TRUE,
                     parscale=rep(1, length=length(start)),
                     ## sumt parameters
@@ -17,13 +15,20 @@ maxBFGS <- function(fn, grad=NULL, hess=NULL,
    ##            bhhh/BHHH  use information equality approach
    ##
    ## ...           further arguments to fn() and grad()
-
+   if(!inherits(control, "MaxControl")) {
+      mControl <- addControlList(maxControl(iterlim=200), control)
+                           # default values
+   }
+   else {
+      mControl <- control
+   }
+   mControl <- maxControl(mControl, ...)
    result <- maxOptim( fn = fn, grad = grad, hess = hess,
       start = start, method = "BFGS", fixed = fixed,
-      print.level = print.level, iterlim = iterlim, constraints = constraints,
-      tol = tol, reltol = reltol,
+                      constraints = constraints,
                       finalHessian=finalHessian,
                       parscale = parscale,
+                      control=mControl,
       ... )
 
    return(result)
