@@ -1,16 +1,12 @@
 
-maxBFGSR <- function(fn, grad=NULL, hess=NULL, start, print.level=0,
-                  tol=1e-8, reltol=sqrt(.Machine$double.eps),
-                  gradtol=1e-6, steptol=1e-10,
-                            lambdatol=1e-6,
-                            qrtol=1e-10,
-                  iterlim=150,
-                  constraints=NULL,
-                      finalHessian=TRUE,
-                  fixed=NULL,
-                  activePar=NULL,
-                  ...) {
-   ## Newton-Raphson maximisation
+maxBFGSR <- function(fn, grad=NULL, hess=NULL, start, 
+                     constraints=NULL,
+                     finalHessian=TRUE,
+                     fixed=NULL,
+                     activePar=NULL,
+                     control=NULL,
+                     ...) {
+   ## Newton-Raphson maximization
    ## Parameters:
    ## fn          - the function to be minimized.  Returns either scalar or
    ##               vector value with possible attributes 
@@ -22,17 +18,18 @@ maxBFGSR <- function(fn, grad=NULL, hess=NULL, start, print.level=0,
    ##                 rows are simply summed (useful for maxBHHH).
    ## hess        - hessian function (numeric used if missing)
    ## start       - initial parameter vector (eventually w/names)
-   ## steptol     - minimum step size
    ## ...         - extra arguments for fn()
-   ## The stopping criteria
-   ## tol         - maximum allowed absolute difference between sequential values
-   ## reltol      - maximum allowed reltive difference (stops if < reltol*(abs(fn) + reltol)
-   ## gradtol     - maximum allowed norm of gradient vector
-   ## iterlim     - maximum # of iterations
-   ## finalHessian  include final Hessian?  As computing final hessian does not carry any extra penalty for NR method, this option is
-   ##               mostly for compatibility reasons with other maxXXX functions.
-   ##               TRUE/something else  include
-   ##               FALSE                do not include
+   ##          The maxControl structure:
+   ##       The stopping criteria
+   ##     tol         - maximum allowed absolute difference between sequential values
+   ##     reltol      - maximum allowed reltive difference (stops if < reltol*(abs(fn) + reltol)
+   ##     gradtol     - maximum allowed norm of gradient vector
+   ##     steptol     - minimum step size
+   ##     iterlim     - maximum # of iterations
+   ##     finalHessian  include final Hessian?  As computing final hessian does not carry any extra penalty for NR method, this option is
+   ##                  mostly for compatibility reasons with other maxXXX functions.
+   ##                  TRUE/something else  include
+   ##                  FALSE                do not include
    ## activePar   - an index vector -- which parameters are taken as
    ##               variable (free).  Other paramters are treated as
    ##               fixed constants
@@ -58,50 +55,60 @@ maxBFGSR <- function(fn, grad=NULL, hess=NULL, start, print.level=0,
    ##             activePar - logical vector, which parameters are active (not constant)
    ## activePar   logical vector, which parameters were treated as free (resp fixed)
    ## iterations  number of iterations
-   ## type        "Newton-Raphson maximisation"
-   argNames <- c( "fn", "grad", "hess", "start", "print.level",
-      "tol", "reltol", "gradtol", "steptol",
-      "iterlim", "activePar", "fixed" )
-   checkFuncArgs( fn, argNames, "fn", "maxNR" )
+   ## type        "Newton-Raphson maximization"
+   ##
+   ## ------------------------------
+   ## Add parameters from ... to control
+   if(!inherits(control, "MaxControl")) {
+      mControl <- addControlList(maxControl(), control)
+   }
+   else {
+      mControl <- control
+   }
+   mControl <- maxControl(mControl, ...)
+   ##
+   argNames <- c(c( "fn", "grad", "hess", "start",
+                 "activePar", "fixed", "control"),
+                 openParam(mControl))
+   checkFuncArgs( fn, argNames, "fn", "maxBFGSR" )
    if( !is.null( grad ) ) {
-      checkFuncArgs( grad, argNames, "grad", "maxNR" )
+      checkFuncArgs( grad, argNames, "grad", "maxBFGSR" )
    }
    if( !is.null( hess ) ) {
-      checkFuncArgs( hess, argNames, "hess", "maxNR" )
+      checkFuncArgs( hess, argNames, "hess", "maxBFGSR" )
    }
    ## establish the active parameters.  Internally, we just use 'activePar'
    fixed <- prepareFixed( start = start, activePar = activePar,
       fixed = fixed )
+   ## chop off the control args from ... and forward the new ...
+   dddot <- list(...)
+   dddot <- dddot[!(names(dddot) %in% openParam(mControl))]
+   cl <- list(start=start,
+              finalHessian=finalHessian,
+              fixed=fixed,
+              control=mControl)
+   if(length(dddot) > 0) {
+      cl <- c(cl, dddot)
+   }
    if(is.null(constraints)) {
-       result <- maxBFGSRCompute(fn=logLikAttr,
-                              fnOrig = fn, gradOrig = grad, hessOrig = hess,
-                              start=start,
-                              print.level=print.level,
-                              tol=tol, reltol=reltol,
-                              gradtol=gradtol, steptol=steptol,
-                                 lambdatol=lambdatol, qrtol=qrtol,
-                              iterlim=iterlim,
-                                  finalHessian=finalHessian,
-                              fixed=fixed,
-                              ...)
+      cl <- c(quote(maxBFGSRCompute),
+              fn=logLikAttr,
+              fnOrig = fn, gradOrig = grad, hessOrig = hess,
+              cl)
+      result <- eval(as.call(cl))
    } else {
       if(identical(names(constraints), c("ineqA", "ineqB"))) {
-         stop("Inequality constraints not implemented for maxNR")
+         stop("Inequality constraints not implemented for maxBFGSR")
       } else if(identical(names(constraints), c("eqA", "eqB"))) {
                            # equality constraints: A %*% beta + B = 0
-         result <- sumt(fn=fn, grad=grad, hess=hess,
-                        start=start,
-                        maxRoutine=maxBFGSR,
-                        constraints=constraints,
-                        print.level=print.level,
-                        tol=tol, reltol=reltol,
-                        gradtol=gradtol, steptol=steptol,
-                        lambdatol=lambdatol, qrtol=qrtol,
-                        iterlim=iterlim,
-                        fixed=fixed,
-                        ...) 
+         cl <- c(quote(sumt),
+                 fn=fn, grad=grad, hess=hess,
+                 maxRoutine=maxBFGSR,
+                 constraints=list(constraints),
+                 cl)
+         result <- eval(as.call(cl))
       } else {
-         stop("maxBFGS only supports the following constraints:\n",
+         stop("maxBFGSR only supports the following constraints:\n",
               "constraints=list(ineqA, ineqB)\n",
               "\tfor A %*% beta + B >= 0 linear inequality constraints\n",
               "current constraints:",
