@@ -51,7 +51,7 @@ summary.maxLik <- function(object, eigentol=1e-12,... ) {
    activePar <- activePar( object )
    if((object$code < 100) & !is.null(coef.maxLik(object))) {
                            # in case of infinity at initial values, the coefs are not provided
-       t <- coef.maxLik(object)/stdEr.maxLik(object)
+       t <- coef.maxLik(object)/stdEr.maxLik(object, eigentol=eigentol)
        p <- 2*pnorm( -abs( t))
        t[!activePar(object)] <- NA
        p[!activePar(object)] <- NA
